package com.perforb.jpaz.extract;

import com.perforb.jpaz.extract.component.Component;

public class ComponentCalculator
{
    /**
     * Singleton instance.
     */
    private static final ComponentCalculator builder = new ComponentCalculator();

    /**
     * Private Constructor.
     */
    private ComponentCalculator()
    {
    }

    /**
     * Return the singleton instance.
     */
    public static ComponentCalculator getInstance()
    {
        return ComponentCalculator.builder;
    }

    public CalculationResult getResult(Component component)
    {
        if (!component.isRoot()) {
            throw new RuntimeException("This component isn't root object.");
        }
        CalculationResult reslut = new CalculationResult();
        reslut.setTotalPackage(calcTotalPackage(component, 0));
        reslut.setTotalClass(calcTotalClass(component, 0));
        reslut.setTotalMethod(calcTotalMethod(component, 0));
        reslut.setMaxLengthOfPackageName(calcMaxLengthOfPackage(component, 0));
        reslut.setMaxLengthOfClassName(calcMaxLengthOfClass(component, 0));
        return reslut;
    }

    /**
     * Returns sum total of packages in the
     * project.
     */
    public int calcTotalPackage(Component component, int count)
    {
        if (component == null || component.isClassFile()) return 0;
        for (Component c1 : component.getChildren()) {
            if (c1.isPackage()) {
                calcTotalPackage(c1, ++count);
            }
        }
        return count;
    }

    /**
     * Returns sum total of classes in the
     * project.
     */
    public int calcTotalClass(Component component, int count)
    {
        if (component == null || component.isClassFile()) return 0;
        for (Component c1 : component.getChildren()) {
            if (c1.isClassFile()) {
                count++;
            }
            if (c1.isPackage()) {
                count = calcTotalClass(c1, count);
            }
        }
        return count;
    }

    /**
     * Returns sum total of methods in the
     * project.
     */
    public int calcTotalMethod(Component component, int count)
    {
        if (component == null || component.isClassFile()) return 0;
        for (Component c1 : component.getChildren()) {
            if (c1.isClassFile()) {
                count += c1.getMethods().length;
            }
            if (c1.isPackage()) {
                count = calcTotalMethod(c1, count);
            }
        }
        return count;
    }

    /**
     * Returns maximal length of package name in
     * the project.
     */
    public int calcMaxLengthOfPackage(Component component, int length)
    {
        if (component == null || component.isClassFile()) return 0;
        int temp = component.getName().length();
        length = (temp > length) ? temp : length;
        for (Component c1 : component.getChildren()) {
            if (c1.isPackage()) {
                length = calcMaxLengthOfPackage(c1, length);
            }
        }
        return length;
    }

    /**
     * Returns maximal length of class name in the
     * project.
     */
    public int calcMaxLengthOfClass(Component component, int length)
    {
        if (component == null || component.isClassFile()) return 0;
        for (Component c1 : component.getChildren()) {
            if (c1.isClassFile()) {
                int temp = c1.getName().length();
                length = (temp > length) ? temp : length;
            }
            if (c1.isPackage()) {
                length = calcMaxLengthOfClass(c1, length);
            }
        }
        return length;
    }

    public CalculationResult sumCalculationResult(CalculationResult... results)
    {
        CalculationResult sum = new CalculationResult();
        for (CalculationResult result : results) {
            sum.setTotalPackage(sum.getTotalPackage() + result.getTotalPackage());
            sum.setTotalClass(sum.getTotalClass() + result.getTotalClass());
            sum.setTotalMethod(sum.getTotalMethod() + result.getTotalMethod());
            int maxlengthOfpkg = result.getMaxLengthOfPackageName();
            int maxlengthOfcls = result.getMaxLengthOfClassName();
            if (sum.getMaxLengthOfPackageName() < maxlengthOfpkg)
                sum.setMaxLengthOfPackageName(maxlengthOfpkg);
            if (sum.getMaxLengthOfClassName() < maxlengthOfcls)
                sum.setMaxLengthOfClassName(maxlengthOfcls);
        }
        return sum;
    }
}
