package com.perforb.jpaz.extract.component;

import java.io.File;
import java.lang.reflect.Method;
import java.util.LinkedList;
import java.util.Queue;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.perforb.jpaz.UnknownPackageException;
import com.perforb.jpaz.common.Util;

public abstract class AbstractComponent implements Component
{
    protected static final Log log    = LogFactory.getLog(AbstractComponent.class);

    /**
     * {@link Package} Object の場合はパッケージ名,
     * {@link ClassFile} Object の場合はクラス名を保持します.
     */
    protected String           name;

    /**
     * ルートコンポーネントであるか.
     */
    private boolean            isRoot = false;

    /**
     * Constructor.
     */
    public AbstractComponent(String name)
    {
        this.name = name;
    }

    /**
     * ルートコンポーネントを構築するためのコンストラクタ.
     * 
     * @param packageName パッケージ名
     * @param isRoot ルートコンポーネントの場合に true をセット
     */
    public AbstractComponent(String name, boolean isRoot)
    {
        this.name = name;
        this.isRoot = isRoot;
    }

    /**
     * 木構造のデータを構築します.本メソッドはルートとなる {@link Package}
     * Object でのみ有効です.
     * 
     * @param classesDir
     * @param rootPackage
     * @return ビルドが成功した場合に true を返します.
     */
    @Override
    public boolean build(File classesDir, String rootPackage, String outlim)
    {
        if (!isRoot() || !isPackage()) {
            log.error("Component is not ROOT or Package.");
            return false;
        }
        int hexadecimal = Integer.decode(outlim);
        Queue<String> packages = new LinkedList<String>();
        setTargetPackages(packages, classesDir, rootPackage);
        while (!packages.isEmpty()) {
            String targetPackage = packages.poll();
            Component pkg = new Package(targetPackage);
            addComponent(pkg);
            File file = Util.getPackageDir(classesDir, targetPackage);
            File[] fileList = file.listFiles();
            for (File target : fileList) {
                if (target.isDirectory()) continue;
                String targetName = target.getName();
                if (Util.isClassFile(targetName)) {
                    String pkgName = pkg.getName();
                    String className = Util.removeSuffix(targetName);
                    Class<?> cl = Util.forName(pkgName + '.' + className);
                    Method[] methods = cl.getDeclaredMethods();
                    if (hexadecimal != 0) methods = excludeMethod(methods, hexadecimal);
                    pkg.addComponent(new ClassFile(className, pkgName, methods));
                }
            }
        }
        return true;
    }

    /**
     * {@link Package} オブジェクトの場合はパッケージ名,
     * {@link ClassFile} オブジェクトの場合はクラス名を返します.
     */
    @Override
    public String getName()
    {
        return name;
    }

    @Override
    public boolean isRoot()
    {
        return isRoot;
    }

    private void setTargetPackages(Queue<String> packages, File classesDir, String rootPackage)
    {
        if (!packages.contains(rootPackage)) {
            packages.offer(rootPackage);
        }
        File file = Util.getPackageDir(classesDir, rootPackage);
        File[] fileList = file.listFiles();
        if (fileList == null) {
            throw new UnknownPackageException("Unknown package: " + rootPackage);
        }
        for (File target : fileList) {
            String targetName = target.getName();
            if (Util.isPackageDir(target)) {
                String packageName = rootPackage + '.' + targetName;
                packages.offer(packageName);
                setTargetPackages(packages, classesDir, packageName);
            }
        }
    }

    private Method[] excludeMethod(Method[] methods, int hexadecimal)
    {
        for (int i = 0; i < methods.length; i++) {
            int mod = methods[i].getModifiers();
            if ((mod & hexadecimal) == 0) {
                methods = (Method[]) ArrayUtils.remove(methods, i--);
            }
        }
        return methods;
    }
}
